@@ -1,6 +1,7 @@
 """
 Mechanisms for discovering information about an OpenID issuer
 """
+
 import requests
 
 from simple_openid_connect import utils
@@ -21,10 +22,8 @@ def discover_configuration_from_issuer(issuer: str) -> ProviderMetadata:
     :raises OpenidProtocolError: When the communication with the provider was not possible or the response was not in an
         expected format
     """
-    if issuer.endswith("/"):
-        config_url = f"{issuer}.well-known/openid-configuration"
-    else:
-        config_url = f"{issuer}/.well-known/openid-configuration"
+    issuer = issuer.rstrip("/")
+    config_url = f"{issuer}/.well-known/openid-configuration"
     response = requests.get(config_url)
 
     if not utils.is_application_json(response.headers["Content-Type"]):
@@ -34,8 +33,8 @@ def discover_configuration_from_issuer(issuer: str) -> ProviderMetadata:
         )
 
     try:
-        result = ProviderMetadata.parse_raw(response.content)
-        assert result.issuer == issuer, "issuer mismatch"
+        result = ProviderMetadata.model_validate_json(response.content)
+        assert result.issuer.rstrip("/") == issuer, "issuer mismatch"
     except Exception as e:
         raise OpenidProtocolError(
             "The provider did not respond with a provider configuration according to spec"
