@@ -5,9 +5,10 @@ The Authorization Server can also authenticate the Client before exchanging the
 
 **The Authorization Code flow is suitable for Clients that can securely maintain a Client Secret between themselves and the Authorization Server.**
 """
+
 import copy
 import logging
-from typing import Literal, Union
+from typing import Literal, Optional, Union
 
 import requests
 from furl import furl
@@ -21,18 +22,31 @@ from simple_openid_connect.data import (
     TokenRequest,
     TokenSuccessResponse,
 )
-from simple_openid_connect.exceptions import AuthenticationFailedError
+from simple_openid_connect.exceptions import AuthenticationFailedError, ValidationError
 
 logger = logging.getLogger(__name__)
 
 
 def start_authentication(
-    authorization_endpoint: str, scope: str, client_id: str, redirect_uri: str
+    authorization_endpoint: str,
+    scope: str,
+    client_id: str,
+    redirect_uri: str,
+    state: Optional[str] = None,
+    nonce: Optional[str] = None,
+    prompt: Optional[list[str]] = None,
+    code_challenge: Optional[str] = None,
+    code_challenge_method: Optional[str] = None,
 ) -> str:
     """
     Start the authentication process by constructing an appropriate :class:`AuthenticationRequest`, serializing it and
     returning a which the end user now needs to visit.
 
+    :param state: The state intended to prevent Cross-Site Request Forgery.
+    :param nonce: String value used to associate a Client session with an ID Token, and to mitigate replay attacks.
+    :param prompt: Specifies whether the Authorization Server prompts the End-User for reauthentication and consent.
+        The defined values are: "none", "login", "consent" and "select_account", multiple may be given as a list.
+
     :returns: A URL to which the user agent should be redirected
     """
     request = AuthenticationRequest(
@@ -40,6 +54,11 @@ def start_authentication(
         client_id=client_id,
         redirect_uri=redirect_uri,
         response_type="code",
+        state=state,
+        nonce=nonce,
+        prompt=prompt,
+        code_challenge=code_challenge,
+        code_challenge_method=code_challenge_method,
     )
     return request.encode_url(authorization_endpoint)
 
@@ -49,6 +68,10 @@ def handle_authentication_result(
     token_endpoint: str,
     client_authentication: ClientAuthenticationMethod,
     redirect_uri: Union[Literal["auto"], str] = "auto",
+    state: Optional[str] = None,
+    code_verifier: Optional[str] = None,
+    code_challenge: Optional[str] = None,
+    code_challenge_method: Optional[str] = None,
 ) -> Union[TokenSuccessResponse, TokenErrorResponse]:
     """
     Handle an authentication result that is communicated to the RP in form of the user agents current url after having started an authentication process via :func:`start_authentication`.
@@ -60,8 +83,10 @@ def handle_authentication_result(
     :param client_authentication: A way for the client to authenticate itself
     :param redirect_uri: The `redirect_uri` that was specified during the authentication initiation.
         If the special value `auto` is used, it is assumed that `current_url` is the that callback and it is stripped of query parameters and fragments to reproduce the originally supplied one.
+    :param state: The `state` that was specified during the authentication initiation.
 
     :raises AuthenticationFailedError: If the current url indicates an authentication failure that prevents an access token from being retrieved.
+    :raises ValidationError: If the returned state does not match the given state.
 
     :returns: The result of the token exchange
     """
@@ -76,17 +101,24 @@ def handle_authentication_result(
             copy.deepcopy(current_furl).remove(fragment=True, query=True)
         )
         logger.debug(
-            f"a redirect_uri value of 'auto' was specified. Reproducing redirect_uri (%s) from current_url (%s)",
+            "a redirect_uri value of 'auto' was specified. Reproducing redirect_uri (%s) from current_url (%s)",
             redirect_uri,
             current_furl,
         )
 
     auth_response_msg = AuthenticationSuccessResponse.parse_url(str(current_furl))
+
+    if state != auth_response_msg.state:
+        raise ValidationError("Returned state does not match given state.")
+
     return exchange_code_for_tokens(
         token_endpoint=token_endpoint,
         authentication_response=auth_response_msg,
         redirect_uri=redirect_uri,
         client_authentication=client_authentication,
+        code_verifier=code_verifier,
+        code_challenge=code_challenge,
+        code_challenge_method=code_challenge_method,
     )
 
 
@@ -95,6 +127,9 @@ def exchange_code_for_tokens(
     authentication_response: AuthenticationSuccessResponse,
     redirect_uri: str,
     client_authentication: ClientAuthenticationMethod,
+    code_verifier: Optional[str] = None,
+    code_challenge: Optional[str] = None,
+    code_challenge_method: Optional[str] = None,
 ) -> Union[TokenSuccessResponse, TokenErrorResponse]:
     """
     Exchange a received code for access, refresh and id tokens.
@@ -116,6 +151,9 @@ def exchange_code_for_tokens(
         redirect_uri=redirect_uri,
         client_id=client_authentication.client_id,
         grant_type="authorization_code",
+        code_verifier=code_verifier,
+        code_challenge=code_challenge,
+        code_challenge_method=code_challenge_method,
     )
     response = requests.post(
         token_endpoint,
@@ -127,6 +165,6 @@ def exchange_code_for_tokens(
     )
 
     if response.status_code == 200:
-        return TokenSuccessResponse.parse_raw(response.content)
+        return TokenSuccessResponse.model_validate_json(response.content)
     else:
-        return TokenErrorResponse.parse_raw(response.content)
+        return TokenErrorResponse.model_validate_json(response.content)
