@@ -2,20 +2,25 @@
 Django AppConfig for this app
 """
 
+import inspect
 import logging
-from typing import Any, Callable, Optional, Union
+from functools import cached_property
+from typing import TYPE_CHECKING, Any, List, Optional, Union
 
 from django.apps import AppConfig, apps
 from django.conf import settings
 from django.core.cache import cache
+from django.core.checks import Error, Warning, register
 from django.core.exceptions import ImproperlyConfigured
 from django.http import HttpRequest
 from django.shortcuts import resolve_url
 from django.utils.module_loading import import_string
-from pydantic import BaseModel
+from pydantic import BaseModel, ConfigDict
 
 from simple_openid_connect.client import OpenidClient
-from simple_openid_connect.data import IdToken
+
+if TYPE_CHECKING:
+    from simple_openid_connect.integrations.django.user_mapping import UserMapper
 
 logger = logging.getLogger(__name__)
 
@@ -25,25 +30,42 @@ class SettingsModel(BaseModel):
     A pydantic model used to validate django settings
     """
 
-    OPENID_ISSUER: str
-    OPENID_CLIENT_ID: str
-    OPENID_CLIENT_SECRET: Optional[str]
-    OPENID_SCOPE: str = "openid"
-    OPENID_REDIRECT_URI: Optional[str] = "simple_openid_connect:login-callback"
-    OPENID_BASE_URI: Optional[str]
-    OPENID_CREATE_USER_FUNC = (
-        "simple_openid_connect.integrations.django.user_mapping.create_user_from_token"
-    )
-    OPENID_UPDATE_USER_FUNC = (
-        "simple_openid_connect.integrations.django.user_mapping.update_user_from_token"
-    )
+    model_config = ConfigDict(from_attributes=True)
 
-    class Config:
-        orm_mode = True
+    OPENID_ISSUER: str
+    "The Openid issuer to use. This is required to be an `https` url and an Openid discovery document should be served under ``{issuer}/.well-known/openid-configuration``."
+
+    OPENID_CLIENT_ID: str
+    "The client id that was issued to you from your OpenId provider."
+
+    OPENID_CLIENT_SECRET: Optional[str] = None
+    "The client secret that was issued to you from your OpenId provider if this is a confidential client."
+
+    OPENID_SCOPE: str = "openid"
+    "The Openid scopes which are requested from the provider when a user logs in. It should be a list of scopes as space separated string and should contain the ``openid`` scope."
+
+    OPENID_REDIRECT_URI: Optional[str] = "simple_openid_connect:login-callback"
+    "The URL or ``named URL pattern <https://docs.djangoproject.com/en/dev/topics/http/urls/#naming-url-patterns>``_ where users are redirected to from the OpenId provider and at which the authentication result is processed.\nThis is sometimes also called a callback uri."
+
+    OPENID_BASE_URI: Optional[str] = None
+    "The absolute base uri of this application. This is used to construct valid redirect urls to the current application."
+
+    OPENID_USER_MAPPER: str = (
+        "simple_openid_connect.integrations.django.user_mapping.UserMapper"
+    )
+    "A string specifying a class that inherits from :class:`simple_openid_connect.integrations.django.user_mapping.UserMapper`."
+
+    OPENID_LOGIN_TIMEOUT: int = 60 * 10
+    "Time in seconds which a login procedure is allowed to take at maximum. If a user takes more than this time between initiating a login and completing it, the login process fails and they have to redo it."
 
 
 class OpenidAppConfig(AppConfig):
+    """
+    The Django AppConfig for simple_openid_connect
+    """
+
     name = "simple_openid_connect.integrations.django"
+    verbose_name = "Simple OpenID Connect"
     label = "simple_openid_connect_django"
 
     default_auto_field = "django.db.models.BigAutoField"
@@ -59,8 +81,7 @@ class OpenidAppConfig(AppConfig):
         # assert that are settings are as required
         try:
             _ = self.safe_settings
-            _ = self.create_user_func
-            _ = self.update_user_func
+            _ = self.user_mapper
         except Exception as e:
             raise ImproperlyConfigured(
                 f"django settings are invalid for openid usage: {e}"
@@ -75,26 +96,27 @@ class OpenidAppConfig(AppConfig):
         assert isinstance(instance, OpenidAppConfig)
         return instance
 
-    @property
+    @cached_property
     def safe_settings(self) -> SettingsModel:
         """
         type-validated version of django settings
         """
-        return SettingsModel.from_orm(settings)
+        return SettingsModel.model_validate(settings)
 
-    @property
-    def create_user_func(self) -> Callable[[IdToken], Any]:
+    @cached_property
+    def user_mapper(self) -> "UserMapper":
         """
-        The function which is configured via django settings and which creates new users based on id tokens.
+        A UserMapper instance of which the actual implementation is configured via django settings.
         """
-        return import_string(self.safe_settings.OPENID_CREATE_USER_FUNC)  # type: ignore
+        from simple_openid_connect.integrations.django.user_mapping import UserMapper
 
-    @property
-    def update_user_func(self) -> Callable[[Any, IdToken], None]:
-        """
-        The function which is configured via django settings and which updates user objects based on id tokens.
-        """
-        return import_string(self.safe_settings.OPENID_UPDATE_USER_FUNC)  # type: ignore
+        cls = import_string(self.safe_settings.OPENID_USER_MAPPER)
+        if not issubclass(cls, UserMapper):
+            raise ImproperlyConfigured(
+                "OPENID_USER_MAPPER setting does not point to a subclass of UserMapper",
+                cls,
+            )
+        return cls()  # type: ignore
 
     def get_client(
         self, own_base_uri: Union[HttpRequest, str, None] = None
@@ -114,7 +136,6 @@ class OpenidAppConfig(AppConfig):
         # use a cached client instance if one exists or create a new one if not
         client = cache.get("openid_client")  # type: OpenidClient
         if client is None:
-
             # determine base_uri of this app
             if self.safe_settings.OPENID_REDIRECT_URI is not None:
                 if self.safe_settings.OPENID_BASE_URI is not None:
@@ -147,3 +168,46 @@ class OpenidAppConfig(AppConfig):
             cache.set("openid_client", client)
 
         return client
+
+
+@register
+def check_middleware(*args: Any, **kwargs: Any) -> List[Union[Warning, Error]]:
+    """
+    Check that our TokenVerificationMiddleware has been added to django settings
+    """
+    from simple_openid_connect.integrations.django.middleware import (
+        TokenVerificationMiddleware,
+    )
+
+    if (
+        hasattr(settings, "OPENID_REDIRECT_URI")
+        and settings.OPENID_REDIRECT_URI is None  # type: ignore
+    ):
+        # redirect uri is set to None,
+        # so the library is probably used as a resource server,
+        # therefore we don't need the middleware
+        return []
+
+    has_token_verification_middleware = False
+    for middleware in settings.MIDDLEWARE:
+        try:
+            middleware_cls = import_string(middleware)
+        except ImportError:
+            continue
+        if inspect.isclass(middleware_cls) and issubclass(
+            middleware_cls, TokenVerificationMiddleware
+        ):
+            has_token_verification_middleware = True
+            break
+
+    if not has_token_verification_middleware:
+        return [
+            Warning(
+                "Access tokens are not checked for expiry and users are not automatically logged out when their access token expires.",
+                hint="Add 'simple_openid_connect.integrations.django.middleware.TokenVerificationMiddleware' to MIDDLEWARE.",
+                obj="simple_openid_connect",
+                id="simple_openid_connect.W001",
+            )
+        ]
+    else:
+        return []
