@@ -46,6 +46,6 @@ def exchange_refresh_token(
     )
 
     if response.status_code == 200:
-        return TokenSuccessResponse.parse_raw(response.content)
+        return TokenSuccessResponse.model_validate_json(response.content)
     else:
-        return TokenErrorResponse.parse_raw(response.content)
+        return TokenErrorResponse.model_validate_json(response.content)
