@@ -1,12 +1,12 @@
 #!/usr/bin/env python
 """Django's command-line utility for administrative tasks."""
-import os
+
 import sys
 
 
 def main():
     """Run administrative tasks."""
-    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "django_test_project.settings")
+    # os.environ.setdefault("DJANGO_SETTINGS_MODULE", "django_test_project.settings")
     try:
         from django.core.management import execute_from_command_line
     except ImportError as exc:
